/* This source file is part of XBiG
 *     (XSLT Bindings Generator)
 * For the latest info, see http://sourceforge.net/projects/xbig/
 * 
 * Copyright (c) 2005 netAllied GmbH, Tettnang
 * Also see acknowledgements in Readme.html
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 * http://www.gnu.org/copyleft/lesser.txt.
 */
package org.xbig.base;

import org.xbig.base.*;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class DirectByteBuffer {

    private boolean mExternalBuffer;
    private ByteBuffer mDirectBuffer;
    private BytePointer mBytePointer;

    private static native Object createBuffer(long size);
    private static native long getBufferAddress(Object buffer);
    private static native void destroyBuffer(Object buffer);

    /**
     * Create a new buffer of given capacity with native byte order.
     * @param capacity The capacity of the byte array to create.
     */
    public DirectByteBuffer(int capacity) throws OutOfMemoryError {
        mDirectBuffer = (ByteBuffer)createBuffer((long)capacity);
        if (mDirectBuffer == null) {
            throw new OutOfMemoryError();
        }
        mExternalBuffer = false;
        mBytePointer = new BytePointer(0);
        mBytePointer.setInstancePointer( getBufferAddress(mDirectBuffer) , true);
    }

    /**
     * Wrap an existing directly allocated ByteBuffer.
     * @param byteBuffer The ByteBuffer to wrap.
     */
    public DirectByteBuffer(ByteBuffer byteBuffer)  {
        if (byteBuffer == null) {
            throw new IllegalArgumentException("byteBuffer cannot be null");
        }
        if (!byteBuffer.isDirect()) {
            throw new IllegalArgumentException("byteBuffer must be direct");
        }
        mDirectBuffer = byteBuffer;
        mExternalBuffer = true;
        mBytePointer = new BytePointer(0);
        mBytePointer.setInstancePointer( getBufferAddress(byteBuffer) , true);
    }

    public int capacity() {
        if (mDirectBuffer == null) {
            return 0;
        }
        return mDirectBuffer.capacity();
    }

    public void destroy() {
        if (mExternalBuffer && mDirectBuffer != null) {
            destroyBuffer(mDirectBuffer);
            mDirectBuffer = null;
            mBytePointer = null;
        }
    }

    /**
     * Pin to get a BytePointer object.
     */
    public BytePointer getBytePointer() {
        if (mDirectBuffer == null) {
            return null;
        }
        return mBytePointer;
    }

    public byte[] getByteArray() {
        if (mDirectBuffer == null) {
            return null;
        }
        return mDirectBuffer.array();
    }

    public ByteBuffer getByteBuffer() {
        return mDirectBuffer;
    }
}
